﻿using System;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using Ares.UDP;
using arca4.Scripting;

namespace arca4
{
    public class Arca
    {
        private Thread server_thread;
        private TcpListener listener;
        private UdpListener udp;
        private bool terminate;

        public void Start()
        {
            this.server_thread = new Thread(new ThreadStart(this.ServerThread));
            this.server_thread.Start();
        }

        public void Stop()
        {
            this.terminate = true;

            if (this.listener != null)
                this.listener.Stop();

            if (this.udp != null)
                this.udp.Stop();

            UserPool.Users.ForEach(x => x.TerminateSocket());
        }

        private void ServerThread()
        {
            this.terminate = false;

            Muzzles.LoadRecords();
            Bans.LoadRecords();
            MOTD.LoadMOTD();
            UdpServerPool.Load();
            ScriptManager.AutoLoad();

            this.listener = new TcpListener(new IPEndPoint(IPAddress.Any, Settings.Port));
            this.listener.Start();

            this.udp = new UdpListener();
            this.udp.Start();

            uint one_second_timer = Helpers.UnixTime;
            uint one_minute_timer = Helpers.UnixTime;

            while (true)
            {
                if (this.terminate)
                    return;

                TimeSpan tspan = Helpers.Timestamp;
                uint time = (uint)tspan.TotalSeconds;

                this.CheckNewUsers(time);
                this.ServiceCurrentUsers(time);
                this.udp.DoTasks(time);
                ScriptTimers.CheckTimers(tspan.TotalMilliseconds);                
                
                if (one_second_timer < time)
                {
                    one_second_timer = time;
                    UserPool.SendFastPings(time);
					ServerEvents.OnTimer();
                }

                if ((one_minute_timer + 60) < time)
                {
                    one_minute_timer = time;
                    FloodProtection.ResetFlooding();
                }

                Thread.Sleep(50);
            }
        }

        private void CheckNewUsers(uint time)
        {
            while (this.listener.Pending())
            {
                UserObject user = new UserObject(this.listener.AcceptSocket(), time);

                if (!this.udp.IsFirewall(user))
                    UserPool.Users.Add(user);
            }
        }

        private void ServiceCurrentUsers(uint time)
        {
            UserPool.Users.ForEach(x => x.SocketTasks(time));
            UserPool.Users.FindAll(x => x.Expired).ForEach(x => x.Disconnect());
            UserPool.Users.RemoveAll(x => x.Expired);
        }

    }
}
