﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Net;
using System.Xml.Linq;
using System.IO;
using Ares.Protocol;

namespace arca4
{
    class Bans
    {
        private static List<UserRecordItem> banItems = new List<UserRecordItem>();

        public static bool IsBanned(UserObject userobj)
        {
            return banItems.Find(x => x.ExternalIP.Equals(userobj.ExternalIP) || x.Guid.Equals(userobj.Guid)) != null;
        }

        public static void AddBan(UserObject userobj)
        {
            banItems.Add(new UserRecordItem(userobj, Helpers.UnixTime));
            UpdateRecords();
        }

        public static String RemoveBan(String name)
        {
            int i = banItems.FindIndex(x => x.Name.StartsWith(name));

            if (i > -1)
            {
                String result = banItems[i].Name;
                banItems.RemoveAt(i);
                UpdateRecords();
                return result;
            }

            return null;
        }

        public static String RemoveBan(int index)
        {
            if (index >= 0 && index < banItems.Count)
            {
                String result = banItems[index].Name;
                banItems.RemoveAt(index);
                UpdateRecords();
                return result;
            }

            return null;
        }

		public static void RemoveBan(UserRecordItem item)
		{
            if (banItems.Remove(item))
                UpdateRecords();
		}

        public static void Clear()
        {
            banItems.Clear();
            UpdateRecords();
        }

        public static void ListBans(UserObject userobj)
        {
            int i = 0;
            banItems.ForEach(x => userobj.SendPacket(AresTCPPackets.NoSuch((i++) + " - " + x.Name + " [" + x.ExternalIP + "]")));

            if (i == 0)
                userobj.SendPacket(AresTCPPackets.NoSuch("no banned users"));
        }

		public static ReadOnlyCollection<UserRecordItem> Items
		{
			get
			{
				return new ReadOnlyCollection<UserRecordItem>(banItems);
			}
		}

        public static void LoadRecords()
        {
            String path = Environment.GetFolderPath(System.Environment.SpecialFolder.ApplicationData);
            path = Path.Combine(path, "Arca4");

            if (!Directory.Exists(path))
                Directory.CreateDirectory(path);

            path = Path.Combine(path, "bans.xml");

            if (File.Exists(path))
            {
                try
                {
                    XDocument xml = XDocument.Load(path);

                    banItems = (from i in xml.Descendants("Ban")
                                select new UserRecordItem(i)).ToList<UserRecordItem>();
                }
                catch { }
            }
        }

        private static void UpdateRecords()
        {
            String path = Environment.GetFolderPath(System.Environment.SpecialFolder.ApplicationData);
            path = Path.Combine(path, "Arca4");

            if (!Directory.Exists(path))
                Directory.CreateDirectory(path);

            path = Path.Combine(path, "bans.xml");

            XDocument xml = new XDocument(new XElement("Bans"));

            foreach (UserRecordItem i in banItems)
                xml.Element("Bans").Add(new XElement("Ban",
                    new XElement("Name", Convert.ToBase64String(Encoding.UTF8.GetBytes(i.Name))),
                    new XElement("Version", Convert.ToBase64String(Encoding.UTF8.GetBytes(i.Version))),
                    new XElement("Guid", Convert.ToBase64String(i.Guid.ToByteArray())),
                    new XElement("ExternalIP", i.ExternalIP.ToString()),
                    new XElement("LocalIP", i.LocalIP.ToString()),
                    new XElement("Port", i.Port.ToString()),
                    new XElement("Time", i.Time.ToString())));

            try
            {
                xml.Save(path);
            }
            catch { }
        }
    }
}
