﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using Ares.IO;

namespace arca4
{
    class FloodProtection
    {
        public static bool IsFlooding(UserObject userobj, ProtoMessage msg, uint time)
        {
            if (time > userobj.LastPacketTime)
            {
                userobj.LastPacketTime = time;
                userobj.PublicTextPacketsReceived = 0;
                userobj.PrivateTextPacketsReceived = 0;
                userobj.OtherPacketsReceived = 0;
            }

            switch (msg)
            {
                case ProtoMessage.MSG_CHAT_CLIENT_ADDSHARE:
                    return userobj.Files.Count > 12000; // browse packets can flood freely but with a cap of 12000

                case ProtoMessage.MSG_CHAT_CLIENT_UPDATE_STATUS:
                case ProtoMessage.MSG_CHAT_ADVANCED_FEATURES_PROTOCOL:
                case ProtoMessage.MSG_CHAT_CLIENT_PUBLIC:
                case ProtoMessage.MSG_CHAT_CLIENT_PVT:
                case ProtoMessage.MSG_CHAT_CLIENT_EMOTE:
                case ProtoMessage.MSG_CHAT_CLIENT_FASTPING:
                    return false;

                default:
                    return ++userobj.OtherPacketsReceived > 8; // might lower this a bit
            }
        }

        private static List<IPAddress> typers = new List<IPAddress>();
        private static List<FloodTextItem> messages = new List<FloodTextItem>();

        public static bool IsTextFlooding(UserObject userobj, String text)
        {
            typers.Add(userobj.ExternalIP);

            if (typers.Count > 8)
                typers.RemoveAt(0);

            if (typers.Count == 8)
                if (typers.FindAll(x => x.Equals(userobj.ExternalIP)).Count == 8) // past 8 messages from the same ip address
                {
                    ResetFlooding();
                    return userobj.Level == 0;
                }

            String str = text.Trim().ToUpper();
            messages.Add(new FloodTextItem { Name = userobj.Name, Text = str });

            if (messages.Count > 4)
                messages.RemoveAt(0);

            if (messages.Count == 4)
                if (messages.FindAll(x => x.Name == userobj.Name && x.Text == str).Count == 4) // typed 4 identical messages
                {
                    ResetFlooding();
                    return userobj.Level == 0;
                }

            if (++userobj.PublicTextPacketsReceived > 3) // typed over 3 messages in 1 second
            {
                ResetFlooding();
                return userobj.Level == 0;
            }

            return false;
        }

        public static void ResetFlooding()
        {
            typers.Clear();
            messages.Clear();
        }

        public static bool IsPMFlooding(UserObject userobj, String text)
        {
            if (userobj.Level > 0)
                return false;

            return ++userobj.PrivateTextPacketsReceived > 5; // typed over 5 messages in 1 second
        }
    }

    class FloodTextItem
    {
        public String Name { get; set; }
        public String Text { get; set; }
    }
}
