﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using Ares.IO;
using Compression;

namespace arca4
{
    class Hashlinks
    {
        public static String EncodeHashlink(ChatroomObject item)
        {
            AresTCPPacketWriter packet = new AresTCPPacketWriter();
            packet.WriteBytes(new byte[20]);
            packet.WriteString("CHATCHANNEL");
            packet.WriteIP(item.ExternalIP);
            packet.WriteUInt16(item.Port);
            packet.WriteIP(item.LocalIP);
            packet.WriteString(item.Name);
            packet.WriteByte(0);
            byte[] bytes = packet.ToByteArray();
            bytes = ZipLib.Compress(bytes);
            bytes = e67(bytes, 28435);
            return Convert.ToBase64String(bytes);
        }

        public static ChatroomObject DecodeHashlink(String item)
        {
            ChatroomObject chitem = new ChatroomObject();

            item = item.Trim();

            if (item.StartsWith("arlnk://"))
                item = item.Substring(8);

            try
            {
                if (item.ToUpper().StartsWith("CHATROOM:")) // not encrypted
                {
                    item = item.Substring(9);
                    int split = item.IndexOf(":");
                    chitem.ExternalIP = IPAddress.Parse(item.Substring(0, split));
                    chitem.LocalIP = chitem.ExternalIP;
                    item = item.Substring(split + 1);
                    split = item.IndexOf("|");
                    chitem.Port = ushort.Parse(item.Substring(0, split));
                    chitem.Name = item.Substring(split + 1);
                    chitem.Topic = String.Empty;
                    return chitem;
                }
                else // encrypted
                {
                    byte[] bytes = Convert.FromBase64String(item);
                    bytes = d67(bytes, 28435);
                    bytes = ZipLib.Decompress(bytes);
                    AresTCPPacketReader packet = new AresTCPPacketReader(bytes);
                    packet.SkipBytes(32);
                    chitem.ExternalIP = packet.ReadIP();
                    chitem.Port = packet.ReadUInt16();
                    chitem.LocalIP = packet.ReadIP();
                    chitem.Name = packet.ReadString();
                    chitem.Topic = String.Empty;
                    return chitem;
                }
            }
            catch // badly formed hashlink
            {
                return null;
            }
        }

        private static byte[] d67(byte[] data, int b)
        {
            byte[] buffer = (byte[])data.Clone();

            for (int i = 0; i < data.Length; i++)
            {
                buffer[i] = (byte)(data[i] ^ b >> 8 & 255);
                b = (b + data[i]) * 23219 + 36126 & 65535;
            }

            return buffer;
        }

        private static byte[] e67(byte[] data, int b)
        {
            byte[] buffer = new byte[data.Length];

            for (int i = 0; i < data.Length; i++)
            {
                buffer[i] = (byte)((data[i] ^ (b >> 8)) & 255);
                b = ((buffer[i] + b) * 23219 + 36126) & 65535;
            }

            return buffer;
        }
    }

    class ChatroomObject
    {
        public String Name { get; set; }
        public String Topic { get; set; }
        public IPAddress ExternalIP { get; set; }
        public IPAddress LocalIP { get; set; }
        public ushort Port { get; set; }
    }
}
