﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.Xml.Linq;
using System.IO;
using Ares.Protocol;

namespace arca4
{
    class Muzzles
    {
        private class MuzzledItem
        {
            public String Name { get; private set; }
            public IPAddress ExternalIP { get; private set; }
            public Guid Guid { get; private set; }

            public MuzzledItem(UserObject userobj)
            {
                this.Name = userobj.Name;
                this.ExternalIP = userobj.ExternalIP;
                this.Guid = userobj.Guid;
            }

            public MuzzledItem(XElement xml)
            {
                this.Name = Encoding.UTF8.GetString(Convert.FromBase64String(xml.Element("Name").Value));
                this.ExternalIP = IPAddress.Parse(xml.Element("ExternalIP").Value);
                this.Guid = new Guid(Convert.FromBase64String(xml.Element("Guid").Value));
            }
        }

        private static List<MuzzledItem> Items = new List<MuzzledItem>();

        public static bool IsMuzzled(UserObject userobj)
        {
            return Items.Find(x => x.ExternalIP.Equals(userobj.ExternalIP) || x.Guid.Equals(userobj.Guid)) != null;
        }

        public static void AddMuzzle(UserObject userobj)
        {
            UserPool.Users.ForEach(x =>
            {
                if (x.LoggedIn && (x.ExternalIP.Equals(userobj.ExternalIP) || x.Guid.Equals(userobj.Guid)))
                {
                    x.SendPacket(AresTCPPackets.NoSuch("You are muzzled"));
                    x.Muzzled = true;
                }
            });

            if (Items.Find(x => x.Guid.Equals(userobj.Guid) || x.ExternalIP.Equals(userobj.ExternalIP)) == null)
            {
                Items.Add(new MuzzledItem(userobj));
                UpdateRecords();
            }
        }

        public static void RemoveMuzzle(UserObject userobj)
        {
            UserPool.Users.ForEach(x =>
            {
                if (x.LoggedIn && (x.ExternalIP.Equals(userobj.ExternalIP) || x.Guid.Equals(userobj.Guid)))
                {
                    x.SendPacket(AresTCPPackets.NoSuch("You are unmuzzled"));
                    x.Muzzled = false;
                }
            });

            if (Items.RemoveAll(x => x.Guid.Equals(userobj.Guid) || x.ExternalIP.Equals(userobj.ExternalIP)) > 0)
                UpdateRecords();
        }

        public static void LoadRecords()
        {
            String path = Environment.GetFolderPath(System.Environment.SpecialFolder.ApplicationData);
            path = Path.Combine(path, "Arca4");

            if (!Directory.Exists(path))
                Directory.CreateDirectory(path);

            path = Path.Combine(path, "muzzles.xml");

            if (File.Exists(path))
            {
                try
                {
                    XDocument xml = XDocument.Load(path);

                    Items = (from i in xml.Descendants("Muzzle")
                             select new MuzzledItem(i)).ToList<MuzzledItem>();
                }
                catch { }
            }
        }

        public static void Clear()
        {
            Items.Clear();
            UpdateRecords();
        }

        private static void UpdateRecords()
        {
            String path = Environment.GetFolderPath(System.Environment.SpecialFolder.ApplicationData);
            path = Path.Combine(path, "Arca4");

            if (!Directory.Exists(path))
                Directory.CreateDirectory(path);

            path = Path.Combine(path, "muzzles.xml");

            XDocument xml = new XDocument(new XElement("Muzzles"));

            foreach (MuzzledItem i in Items)
                xml.Element("Muzzles").Add(new XElement("Muzzle",
                    new XElement("Name", Convert.ToBase64String(Encoding.UTF8.GetBytes(i.Name))),
                    new XElement("ExternalIP", i.ExternalIP.ToString()),
                    new XElement("Guid", Convert.ToBase64String(i.Guid.ToByteArray()))));

            try
            {
                xml.Save(path);
            }
            catch { }
        }
    }
}
