﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics.Contracts;
using smnetjs;

namespace arca4.Scripting
{
	class EventItem : IDisposable
	{
		public string ScriptName { get; private set; }
		public EventType EventType { get; private set; }
		public SMFunction CallBack { get; private set; }

		[ContractInvariantMethod]
		private void ObjectInvariant()
		{
			Contract.Invariant(!String.IsNullOrEmpty(ScriptName));
			Contract.Invariant(this.CallBack != null);
		}

		public EventItem(string scriptName, EventType eventType, SMFunction callback)
		{
			Contract.Requires(string.IsNullOrWhiteSpace(scriptName) == false);
			Contract.Requires(callback != null);

			ScriptName = scriptName;
			EventType = eventType;
			CallBack = callback;
		}

		public void Dispose()
		{
			CallBack.Dispose();
		}
	}
}
