﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics.Contracts;
using smnetjs;

namespace arca4.Scripting
{
	static class EventManager
	{

		static List<EventItem> events;

		static EventManager()
		{
			events = new List<EventItem>();
		}



		public static void RegisterEvent(string script, EventType @event, SMFunction method)
		{
			Contract.Requires(string.IsNullOrWhiteSpace(script) == false);
			Contract.Requires(method != null);

			EventItem item = events.Find(c => c.ScriptName == script && c.EventType == @event);

			if (item != null)
			{
				events.Remove(item);
				item.Dispose();
				item = null;
			}
			

			events.Add(new EventItem(script, @event, method));
		}

		public static void UnRegisterEvent(string script, EventType @event)
		{
			Contract.Requires(string.IsNullOrWhiteSpace(script) == false);

			EventItem item = events.Find(c => c.ScriptName == script && c.EventType == @event);

			if (item != null)
			{
				events.Remove(item);
				item.Dispose();
				item = null;
			}
		}

		public static void UnRegisterEvents(string script)
		{
			Contract.Requires(string.IsNullOrWhiteSpace(script) == false);

			var items = events.FindAll(c => c.ScriptName == script);

			if (items != null && items.Count > 0)
			{
				items.ForEach(delegate(EventItem c) { 
					events.Remove(c);
					c.Dispose();
				});

				items = null;
			}
		}

		public static void InvokeEvent(EventType @event, params object[] args)
		{
			events.ForEach(delegate(EventItem c)
			{
				if (c != null && c.EventType == @event)
					c.CallBack.Invoke(args);
			});
		}

		public static T InvokeEvent<T>(EventType @event, params object[] args)
		{
			T ret = default(T);

			if (ret is bool)
			{
				ret = (T)(object)true;
			}

			events.ForEach(delegate(EventItem c){
				if (c == null || c.EventType != @event)
					return;


                if (ret is bool)
                {
                    ret = (T)(object)(Convert.ToBoolean(ret) & Convert.ToBoolean(c.CallBack.Invoke(args)));

                    if (!(bool)(object)ret)
                        return; // don't let another script overwrite a false !!
                }
                else ret = (T)(object)(Convert.ToString(c.CallBack.Invoke(args)));
			});	


			return ret;
		}

	}
}
