﻿using System;
using System.Linq;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.IO;
using System.Diagnostics.Contracts;
using smnetjs;

namespace arca4.Scripting.JS
{
    [SMEmbedded(AllowInheritedMembers = false)]
    class Avatar
    {
        internal Avatar(byte[] img)
        {
            this.ToByteArray = img;
        }

        public Avatar(Image img)
        {
            Contract.Requires(img != null);
            Contract.Requires(img.ImageData != null);

            using (MemoryStream ms = new MemoryStream(img.ImageData))
            {
                using (Bitmap raw = new Bitmap(ms))
                {
                    using (Bitmap bmp = new Bitmap(48, 48))
                    {
                        using (Graphics g = Graphics.FromImage(bmp))
                        {
                            g.InterpolationMode = InterpolationMode.HighQualityBicubic;
                            g.CompositingQuality = CompositingQuality.HighQuality;
                            g.SmoothingMode = SmoothingMode.HighQuality;

                            int _w, _h, _sx, _sy;
                            float _wp, _hp;

                            if (raw.Width > raw.Height)
                            {
                                _w = bmp.Width;
                                _wp = (float)bmp.Width / raw.Width;
                                _h = Convert.ToInt32(raw.Height * _wp);
                            }
                            else
                            {
                                _h = bmp.Height;
                                _hp = (float)bmp.Height / raw.Height;
                                _w = Convert.ToInt32(raw.Width * _hp);
                            }

                            if (_w != _h)
                            {
                                if (_w > _h)
                                {
                                    _sx = 0;
                                    _sy = (bmp.Height - _h) / 2; // - 1
                                }
                                else
                                {
                                    _sx = (bmp.Width - _w) / 2;
                                    _sy = 0;
                                }

                                g.Clear(Color.White);
                            }
                            else
                            {
                                _sx = 0;
                                _sy = 0;
                            }

                            g.DrawImage(raw, new Rectangle(_sx, _sy, _w, _h));

                            ImageCodecInfo info = ImageCodecInfo.GetImageEncoders().First(i => i.MimeType == "image/jpeg");
                            EncoderParameters encoding = new EncoderParameters();
                            encoding.Param[0] = new EncoderParameter(System.Drawing.Imaging.Encoder.Quality, 100L);

                            using (MemoryStream buf = new MemoryStream())
                            {
                                bmp.Save(buf, info, encoding);
                                this.ToByteArray = buf.ToArray();
                            }
                        }
                    }
                }
            }
        }

        internal byte[] ToByteArray { get; private set; }
    }
}
