﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using smnetjs;

namespace arca4.Scripting.JS
{
    [SMEmbedded(AccessibleName = "File")]
    static class DataFile
    {
        [SMMethod(Script = true)]
        public static string load(SMScript script, String filename)
        {
            if (String.IsNullOrEmpty(filename))
                return null;

            try
            {
                foreach (char c in Path.GetInvalidFileNameChars())
                    if (filename.Contains(c))
                        return null;

                String path = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData);
                path = Path.Combine(path, "Arca4", "Data", script.Name);

                if (!Directory.Exists(path))
                    return null;

                path = Path.Combine(path, filename);

                if (!File.Exists(path))
                    return null;

                String str = File.ReadAllText(path);
                return str;
            }
            catch { }

            return null;
        }

        [SMMethod(Script = true)]
        public static bool save(SMScript script, String filename, String text)
        {
            if (String.IsNullOrEmpty(filename))
                return false;

            if (text == null)
                return false;

            try
            {
                foreach (char c in Path.GetInvalidFileNameChars())
                    if (filename.Contains(c))
                        return false;

                String path = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData);
                path = Path.Combine(path, "Arca4", "Data", script.Name);

                if (!Directory.Exists(path))
                    Directory.CreateDirectory(path);

                path = Path.Combine(path, filename);

                File.WriteAllText(path, text);

                return true;
            }
            catch { }

            return false;
        }

        [SMMethod(Script = true)]
        public static bool delete(SMScript script, String filename)
        {
            if (String.IsNullOrEmpty(filename))
                return false;

            try
            {
                foreach (char c in Path.GetInvalidFileNameChars())
                    if (filename.Contains(c))
                        return false;

                String path = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData);
                path = Path.Combine(path, "Arca4", "Data", script.Name);

                if (!Directory.Exists(path))
                    return false;

                path = Path.Combine(path, filename);

                if (!File.Exists(path))
                    return false;

                File.Delete(path);
                return true;
            }
            catch { }

            return false;
        }

        [SMMethod(Script = true)]
        public static bool append(SMScript script, String filename, String text)
        {
            if (String.IsNullOrEmpty(filename))
                return false;

            if (text == null)
                return false;

            try
            {
                foreach (char c in Path.GetInvalidFileNameChars())
                    if (filename.Contains(c))
                        return false;

                String path = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData);
                path = Path.Combine(path, "Arca4", "Data", script.Name);

                if (!Directory.Exists(path))
                    Directory.CreateDirectory(path);

                path = Path.Combine(path, filename);

                bool exists = File.Exists(path);

                using (StreamWriter writer = exists ? File.AppendText(path) : File.CreateText(path))
                    writer.Write(text);

                return true;
            }
            catch { }

            return false;
        }

        [SMMethod(Script = true)]
        public static bool exists(SMScript script, String filename)
        {
            if (String.IsNullOrEmpty(filename))
                return false;

            try
            {
                foreach (char c in Path.GetInvalidFileNameChars())
                    if (filename.Contains(c))
                        return false;

                String path = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData);
                path = Path.Combine(path, "Arca4", "Data", script.Name);

                if (!Directory.Exists(path))
                    return false;

                path = Path.Combine(path, filename);

                return File.Exists(path);
            }
            catch { }

            return false;
        }
    }
}
