﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Ares.Protocol;
using smnetjs;
using System.Diagnostics.Contracts;

namespace arca4.Scripting.JS
{
	sealed class Global
	{

		[SMMethod(Name = "print")]
		public static void Print(string obj)
		{
			Contract.Requires(string.IsNullOrWhiteSpace(obj) == false);

			UserPool.Broadcast(AresTCPPackets.NoSuch(obj));
		}

		[SMMethod(Name = "print")]
		public static void Print(ushort id, string obj)
		{
			Contract.Requires(string.IsNullOrWhiteSpace(obj) == false);

			UserPool.BroadcastToVroom(id, AresTCPPackets.NoSuch(obj)); ;
		}

		[SMMethod(Name = "print")]
		public static void Print(UserObj userobj, string obj)
		{
			Contract.Requires(userobj != null);
			Contract.Requires(userobj.LoggedIn);
			Contract.Requires(string.IsNullOrWhiteSpace(obj) == false);
			
			UserObject user = UserPool.Users.Find(c => c.ID == userobj.ID);
			if (user != null)
				user.SendPacket(AresTCPPackets.NoSuch(obj));
		}

		[SMMethod(Name = "user")]
		public static UserObj User(int id)
		{
            UserObject userobj = UserPool.Users.Find(x => x.ID == id);
            return userobj == null ? null : userobj.ScriptUserObj;
		}

		[SMMethod(Name = "user")]
		public static UserObj User(string name)
		{
			Contract.Requires(string.IsNullOrWhiteSpace(name) == false);

            UserObject userobj = UserPool.Users.Find(x => x.Name == name);
            return userobj == null ? null : userobj.ScriptUserObj;
		}

		[SMMethod(Name = "firstUser")]
		public static UserObj FirstUser()
		{
			UserObject user = UserPool.Users.Where(c => c.LoggedIn).FirstOrDefault();
			return user == null ? null : user.ScriptUserObj;
		}

		[SMMethod(Script = true, Name="scriptName")]
		public static string ScriptName(SMScript script)
		{
			Contract.Requires(script != null);

			return script.Name;
			
		}

		[SMMethod(Name = "sendPM")]
		public static void sendPM(UserObj userobj, string name, string message)
		{
			Contract.Requires(userobj != null);
			Contract.Requires(userobj.LoggedIn);

			Contract.Requires(string.IsNullOrWhiteSpace(name) == false);

			Contract.Requires(string.IsNullOrWhiteSpace(message) == false);

			userobj.InternalUserObject.SendPacket(AresTCPPackets.Private(name, message));
		}

        // timers

        [SMMethod(Name = "setTimeout", Script = true)]
        public static int SetTimeout(SMScript script, SMFunction source, int interval)
        {
            Contract.Requires(script != null);
            Contract.Requires(source != null);

            return ScriptTimers.SetTimeout(script.Name, source, interval);
        }

        [SMMethod(Name = "setTimeout", Script = true)]
        public static int SetTimeout(SMScript script, String source, int interval)
        {
            Contract.Requires(script != null);
            Contract.Requires(!String.IsNullOrWhiteSpace(source));

            return ScriptTimers.SetTimeout(script.Name, source, interval);
        }

        [SMMethod(Name = "clearTimeout", Script = true)]
        public static bool ClearTimeout(SMScript script, int ident)
        {
            Contract.Requires(script != null);

            return ScriptTimers.ClearTimeout(script.Name, ident);
        }

        [SMMethod(Name = "setInterval", Script = true)]
        public static int SetInterval(SMScript script, SMFunction source, int interval)
        {
            Contract.Requires(script != null);
            Contract.Requires(source != null);

            return ScriptTimers.SetInterval(script.Name, source, interval);
        }

        [SMMethod(Name = "setInterval", Script = true)]
        public static int SetInterval(SMScript script, String source, int interval)
        {
            Contract.Requires(script != null);
            Contract.Requires(!String.IsNullOrWhiteSpace(source));

            return ScriptTimers.SetInterval(script.Name, source, interval);
        }

        [SMMethod(Name = "clearInterval", Script = true)]
        public static bool ClearInterval(SMScript script, int ident)
        {
            Contract.Requires(script != null);

            return ScriptTimers.ClearInterval(script.Name, ident);
        }
	}
}
