﻿using System;
using System.Linq;
using System.Threading;
using System.IO;
using System.Net;
using System.Diagnostics.Contracts;
using smnetjs;

namespace arca4.Scripting.JS
{
    [SMEmbedded(AllowInheritedMembers = false)]
    class Image
    {
        private bool busy { get; set; }

        internal byte[] ImageData { get; private set; }

        public String src { get; set; }
        public bool available { get; private set; }

        public SMFunction oncomplete { get; set; }
        public bool completed { get; private set; }


        public void download()
        {
            if (!this.busy)
            {
                Contract.Requires(!String.IsNullOrWhiteSpace(this.src));

                if (this.src.StartsWith("data:"))
                {
                    String data_uri = this.src.Substring(5);
                    this.completed = false;
                    this.available = false;
                    this.ImageData = new byte[] { };

                    foreach (String str in data_uri.Split(new String[] { ";" }, StringSplitOptions.RemoveEmptyEntries))
                        if (str.StartsWith("base64,"))
                        {
                            data_uri = str.Substring(7);
                            this.ImageData = Convert.FromBase64String(data_uri);
                            this.available = true;
                            break;
                        }

                    if (this.ImageData.Length == 0)
                        this.ImageData = null;

                    this.completed = true;

                    if (this.oncomplete != null)
                        this.oncomplete.Invoke(this);
                }
                else
                {
                    Thread thread = new Thread(() =>
                    {
                        this.busy = true;

                        try
                        {
                            this.completed = false;
                            this.available = false;
                            this.ImageData = new byte[] { };

                            WebRequest request = WebRequest.Create(this.src);
                            HttpWebResponse response = (HttpWebResponse)request.GetResponse();
                            bool valid = false;

                            for (int i = 0; i < response.Headers.Count; i++)
                            {
                                String key = response.Headers.Keys[i].ToUpper();
                                String value = response.Headers[i].ToUpper();

                                if (key == "CONTENT-TYPE")
                                    if (value.StartsWith("IMAGE/"))
                                        valid = true;
                            }

                            if (valid)
                            {
                                using (Stream stream = response.GetResponseStream())
                                {
                                    byte[] buf = new byte[1024];
                                    int size = 0;

                                    while ((size = stream.Read(buf, 0, 1024)) > 0)
                                        this.ImageData = this.ImageData.Concat(buf.Take(size)).ToArray();
                                }

                                this.available = true;
                            }

                            response.Close();
                        }
                        catch { }

                        if (this.ImageData.Length == 0)
                            this.ImageData = null;

                        this.busy = false;
                        this.completed = true;

                        if (this.oncomplete != null)
                            this.oncomplete.Invoke(this);
                    });

                    thread.Start();
                }
            }
        }
    }
}
