﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.IO;
using System.Diagnostics.Contracts;
using smnetjs;
using Compression;
using Ares.Protocol;

namespace arca4.Scripting.JS
{
    [SMEmbedded(AllowInheritedMembers = false)]
    class Scribble
    {
        internal List<byte[]> Packets { get; private set; }

        public Scribble(Image img)
        {
            Contract.Requires(img != null);
            Contract.Requires(img.ImageData != null);

            using (MemoryStream ms = new MemoryStream(img.ImageData))
            {
                using (Bitmap raw = new Bitmap(ms))
                {
                    int width = raw.Width;
                    int height = raw.Height;

                    if (width > 384)
                    {
                        width = 384;
                        height = raw.Height - (int)Math.Floor(Math.Floor((double)raw.Height / 100) * Math.Floor(((double)(raw.Width - 384) / raw.Width) * 100));
                    }

                    if (height > 384)
                    {
                        width -= (int)Math.Floor(Math.Floor((double)width / 100) * Math.Floor(((double)(height - 384) / height) * 100));
                        height = 384;
                    }

                    using (Bitmap bmp = new Bitmap(width, height))
                    {
                        using (Graphics g = Graphics.FromImage(bmp))
                        {
                            g.InterpolationMode = InterpolationMode.HighQualityBicubic;
                            g.CompositingQuality = CompositingQuality.HighQuality;
                            g.SmoothingMode = SmoothingMode.HighQuality;
                            g.DrawImage(raw, new Rectangle(0, 0, width, height));

                            ImageCodecInfo info = ImageCodecInfo.GetImageEncoders().First(i => i.MimeType == "image/jpeg");
                            EncoderParameters encoding = new EncoderParameters();
                            encoding.Param[0] = new EncoderParameter(System.Drawing.Imaging.Encoder.Quality, 100L);

                            using (MemoryStream buf = new MemoryStream())
                            {
                                this.Packets = new List<byte[]>();
                                bmp.Save(buf, info, encoding);
                                List<byte> list = new List<byte>(ZipLib.Compress(buf.ToArray()));

                                if (list.Count < 4000)
                                    this.Packets.Add(CustomPackets.CustomData(Settings.BotName, "cb0t_scribble_once", list.ToArray()));
                                else
                                {
                                    List<byte[]> p = new List<byte[]>();

                                    while (list.Count > 4000)
                                    {
                                        p.Add(list.GetRange(0, 4000).ToArray());
                                        list.RemoveRange(0, 4000);
                                    }

                                    if (list.Count > 0)
                                        p.Add(list.ToArray());

                                    for (int i = 0; i < p.Count; i++)
                                    {
                                        if (i == 0)
                                            this.Packets.Add(CustomPackets.CustomData(Settings.BotName, "cb0t_scribble_first", p[i]));
                                        else if (i == (p.Count - 1))
                                            this.Packets.Add(CustomPackets.CustomData(Settings.BotName, "cb0t_scribble_last", p[i]));
                                        else
                                            this.Packets.Add(CustomPackets.CustomData(Settings.BotName, "cb0t_scribble_chunk", p[i]));
                                    }

                                    p.Clear();
                                }

                                list.Clear();
                            }
                        }
                    }
                }
            }
        }
    }
}
