﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics.Contracts;
using System.IO;
using smnetjs;

namespace arca4.Scripting.JS
{
	[SMEmbedded(Name="Script")]
	static class Script
	{
		[SMMethod(Script = true)]
		public static bool create(SMScript script, string name)
		{
            if (String.IsNullOrEmpty(name))
                return false;

            if (name == script.Name)
                return false;

            foreach (char c in Path.GetInvalidFileNameChars())
                if (name.Contains(c))
                    return false;

			ScriptManager.Items.Add(name);
            return true;
		}

		[SMMethod(Script = true)]
		public static string eval(SMScript script, string name, string text)
		{
			Contract.Requires(!String.IsNullOrEmpty(name));

			if (text == null)
				return null;

			ScriptObj scriptObj = ScriptManager.Items.Find(name);

			return scriptObj == null ? null : scriptObj.Eval(text);
		}

        [SMMethod(Script = true)]
        public static bool include(SMScript script, string filename)
        {
            if (String.IsNullOrEmpty(filename))
                return false;

            if (!filename.EndsWith(".js"))
                return false;

            try
            {
                foreach (char c in Path.GetInvalidFileNameChars())
                    if (filename.Contains(c))
                        return false;

                String path = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData);
                path = Path.Combine(path, "Arca4", "Scripts");

                if (!Directory.Exists(path))
                    return false;

                String tmp = path;
                path = Path.Combine(path, filename);

                if (Path.GetDirectoryName(path) != tmp)
                    return false;

                if (!File.Exists(path))
                    return false;

                String src = File.ReadAllText(path);
                bool success = false;

                if (script != null)
                    script.Eval(src, out success);

                return success;
            }
            catch { }

            return false;
        }

        [SMMethod(Script = true)]
        public static bool load(SMScript script, string filename)
        {
            if (String.IsNullOrEmpty(filename))
                return false;

            if (filename == script.Name)
                return false;

            if (!filename.EndsWith(".js"))
                return false;

            try
            {
                foreach (char c in Path.GetInvalidFileNameChars())
                    if (filename.Contains(c))
                        return false;

                String path = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData);
                path = Path.Combine(path, "Arca4", "Scripts");

                if (!Directory.Exists(path))
                    return false;

                String tmp = path;
                path = Path.Combine(path, filename);

                if (Path.GetDirectoryName(path) != tmp)
                    return false;

                if (!File.Exists(path))
                    return false;

                String src = File.ReadAllText(path);
                bool success = false;

                create(script, filename);

                ScriptObj obj = ScriptManager.Items.Find(x => x.ScriptName == filename);
                obj.Script.Eval(src, out success);

                return success;
            }
            catch { }

            return false;
        }

        [SMMethod(Script = true)]
        public static bool kill(SMScript script, string filename)
        {
            if (String.IsNullOrEmpty(filename))
                return false;

            if (filename == script.Name)
                return false;

            ScriptManager.Items.Remove(filename);
            return true;
        }

        [SMMethod(Script = true)]
        public static bool write(SMScript script, string filename, string src)
        {
            if (String.IsNullOrEmpty(filename))
                return false;

            if (String.IsNullOrEmpty(src))
                return false;

            if (!filename.EndsWith(".js"))
                return false;

            try
            {
                foreach (char c in Path.GetInvalidFileNameChars())
                    if (filename.Contains(c))
                        return false;

                String path = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData);
                path = Path.Combine(path, "Arca4", "Scripts");

                if (!Directory.Exists(path))
                    Directory.CreateDirectory(path);

                String tmp = path;
                path = Path.Combine(path, filename);

                if (Path.GetDirectoryName(path) != tmp)
                    return false;

                File.WriteAllText(path, src);
                return true;
            }
            catch { }

            return false;
        }
	}
}
