﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Security.Cryptography;
using System.Diagnostics.Contracts;

using smnetjs;
namespace arca4.Scripting.JS
{
	[SMEmbedded(Name = "Arca4User", AllowInheritedMembers=false)]
	class UserObj
	{
		
		internal UserObject InternalUserObject { get; set;}

		[SMIgnore]
		public UserObj(UserObject userobj)
		{
			Contract.Requires(userobj != null);
			this.InternalUserObject = userobj;
		}

		#region Methods

        [SMMethod(Name = "scribble")]
        public void Scribble(Scribble scribble)
        {
            foreach (byte[] p in scribble.Packets)
                InternalUserObject.SendPacket(p);
        }

		[SMMethod(Name = "ban")]
		public void Ban()
		{
			Bans.AddBan(InternalUserObject);
			InternalUserObject.Disconnect();
		}

		[SMMethod(Name = "clearAvatar")]
		public void ClearAvatar()
		{
			InternalUserObject.Avatar = null;
		}

		[SMMethod(Name = "disconnect")]
		public void Disconnect()
		{
			InternalUserObject.Disconnect();
		}

		[SMMethod(Name = "redirect")]
		public void Redirect(String hashlink)
		{
			InternalUserObject.Redirect(hashlink);
		}

		[SMMethod(Name = "nextUser")]
		public UserObj NextUser()
		{
			UserObject user = UserPool.Users.Where(c => c.ID > this.ID && c.LoggedIn).FirstOrDefault();
			return user == null ? null : user.ScriptUserObj;
		}


		#endregion

		#region Read Only Properties

		[SMProperty(Name = "age")]
		public byte Age
		{
			get { return InternalUserObject.Age; }
		}

		[SMProperty(Name = "browsable")]
		public bool CanBrowse
		{
			get { return InternalUserObject.CanBrowse; }
		}

		[SMProperty(Name = "canScript")]
		public bool CanScript
		{
			get
			{
				return (InternalUserObject.Level >= Settings.ScriptLevel);
			}
		}

		[SMProperty(Name = "county")]
		public byte Country
		{
			get { return InternalUserObject.Country; }
		}

		[SMProperty(Name = "currentQueued")]
		public byte CurrentQueued
		{
			get { return InternalUserObject.CurrentQueued; }
		}

		[SMProperty(Name = "currentUploads")]
		public byte CurrentUploads
		{
			get { return InternalUserObject.CurrentUploads; }
		}

		[SMProperty(Name = "externalIp")]
		public string ExternalIP
		{
			get { return InternalUserObject.ExternalIP.ToString(); }
		}

		[SMProperty(Name = "fastPing")]
		public bool FastPing
		{
			get { return InternalUserObject.FastPing; }
		}

		[SMProperty(Name = "fileCount")]
		public ushort FileCount
		{
			get { return InternalUserObject.FileCount; }
		}

		[SMProperty(Name = "guid")]
		public string Guid
		{
			get
			{

				return Convert.ToBase64String(SHA1.Create().ComputeHash(InternalUserObject.Guid.ToByteArray()));
			}
		}

		[SMProperty(Name = "id")]
		public int ID
		{
			get { return InternalUserObject.ID; }
		}

		[SMProperty(Name = "localIp")]
		public string LocalIP
		{
			get { return InternalUserObject.LocalIP.ToString(); }
		}

		[SMProperty(Name = "location")]
		public string Location
		{
			get { return InternalUserObject.Location; }
		}

		[SMProperty(Name = "isLoggedIn")]
		public bool LoggedIn
		{
			get { return InternalUserObject.LoggedIn; }
		}

		[SMProperty(Name = "maxUploads")]
		public byte MaxUploads
		{
			get { return InternalUserObject.MaxUploads; }
		}

		[SMProperty(Name = "nodeIp")]
		public string NodeIP
		{
			get { return InternalUserObject.NodeIP.ToString(); }
		}

		[SMProperty(Name = "nodePort")]
		public ushort NodePort
		{
			get { return InternalUserObject.NodePort; }
		}

		[SMProperty(Name = "orgName")]
		public string OrgName
		{
			get { return InternalUserObject.OrgName; }
		}

		[SMProperty(Name = "port")]
		public ushort Port
		{
			get { return InternalUserObject.Port; }
		}

		[SMProperty(Name = "isRegistered")]
		public bool Registered
		{
			get { return InternalUserObject.Registered; }
		}

		[SMProperty(Name = "sex")]
		public byte Sex
		{
			get { return InternalUserObject.Sex; }
		}

		[SMProperty(Name = "version")]
		public string Version
		{
			get { return InternalUserObject.Version; }
		}
		
		#endregion
		
		#region Read / Write Properties

        [SMProperty(Name = "avatar")]
        public Avatar Avatar
        {
            get { return InternalUserObject.Avatar == null ? null : new Avatar(InternalUserObject.Avatar); }
            set
            {
                if (value != null)
                    if (value.ToByteArray != null)
                        InternalUserObject.Avatar = value.ToByteArray;
            }
        }

		[SMProperty(Name = "customName")]
		public string CustomName
		{
			get { return InternalUserObject.CustomName; }
			set { InternalUserObject.CustomName = value; }
		}

		[SMProperty(Name = "level")]
		public byte Level
		{
			get { return InternalUserObject.Level; }
			set { InternalUserObject.Level = value; }

		}

		[SMProperty(Name = "isMuzzled")]
		public bool Muzzled
		{
			get
			{
				return InternalUserObject.Muzzled;
			}
			set
			{
				InternalUserObject.Muzzled = value;

                if (value)
                    Muzzles.AddMuzzle(InternalUserObject);
                else
                    Muzzles.RemoveMuzzle(InternalUserObject);
			}
		}

		[SMProperty(Name = "name")]
		public string Name
		{
			get
			{
				return InternalUserObject.Name;
			}
			set
			{
				InternalUserObject.Name = value;
			}
		}

		[SMProperty(Name = "personMessage")]
		public string PersonalMessage
		{
			get { return InternalUserObject.PersonalMessage; }
			
		}

		[SMProperty(Name = "vroom")]
		public ushort Vroom
		{
			get
			{
				return InternalUserObject.Vroom;
			}
			set
			{
				InternalUserObject.Vroom = value;
			}
		}
		
		#endregion
	}
}
