﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using smnetjs;

namespace arca4.Scripting
{
    class ScriptTimers
    {
        private static List<ScriptTimerObj> timers;
        private static int ident_cookie;

        static ScriptTimers()
        {
            timers = new List<ScriptTimerObj>();
            ident_cookie = 0;
        }

        public static int SetTimeout(String script, SMFunction source, int interval)
        {
            if (interval < 50)
                return -1;

            ScriptTimerObj obj = new ScriptTimerObj
            {
                Ident = ident_cookie++,
                Interval = interval,
                Script = script,
                SetTimeout = true,
                Source = source,
                Time = Now,
                IsFunction = true
            };

            timers.Add(obj);
            return obj.Ident;
        }

        public static int SetTimeout(String script, String source, int interval)
        {
            if (interval < 50)
                return -1;

            ScriptTimerObj obj = new ScriptTimerObj
            {
                Ident = ident_cookie++,
                Interval = interval,
                Script = script,
                SetTimeout = true,
                Source = source,
                Time = Now,
                IsFunction = false
            };

            timers.Add(obj);
            return obj.Ident;
        }

        public static bool ClearTimeout(String script, int ident)
        {
            int result = timers.RemoveAll(x => x.Script == script && x.Ident == ident && x.SetTimeout);
            return result > 0;
        }

        public static int SetInterval(String script, SMFunction source, int interval)
        {
            if (interval < 50)
                return -1;

            ScriptTimerObj obj = new ScriptTimerObj
            {
                Ident = ident_cookie++,
                Interval = interval,
                Script = script,
                SetTimeout = false,
                Source = source,
                Time = Now,
                IsFunction = true
            };

            timers.Add(obj);
            return obj.Ident;
        }

        public static int SetInterval(String script, String source, int interval)
        {
            if (interval < 50)
                return -1;

            ScriptTimerObj obj = new ScriptTimerObj
            {
                Ident = ident_cookie++,
                Interval = interval,
                Script = script,
                SetTimeout = false,
                Source = source,
                Time = Now,
                IsFunction = false
            };

            timers.Add(obj);
            return obj.Ident;
        }

        public static bool ClearInterval(String script, int ident)
        {
            int result = timers.RemoveAll(x => x.Script == script && x.Ident == ident && !x.SetTimeout);
            return result > 0;
        }

        public static void CheckTimers(double time)
        {
            for (int i = (timers.Count - 1); i > -1; i--)
            {
                ScriptTimerObj t = timers[i];

                if (time >= (t.Time + t.Interval))
                {
                    if (t.IsFunction)
                        ((SMFunction)t.Source).Invoke();
                    else
                    {
                        ScriptObj obj = ScriptManager.Items[t.Script];
                        obj.Eval((String)t.Source);
                    }

                    if (t.SetTimeout)
                        timers.RemoveAt(i);
                    else
                        timers[i].Time = time;
                }
            }
        }

        public static void KillTimers(String script)
        {
            timers.RemoveAll(x => x.Script == script);
        }

        private static double Now
        {
            get
            {
                TimeSpan ts = (DateTime.UtcNow - new DateTime(1970, 1, 1, 0, 0, 0));
                return ts.TotalMilliseconds;
            }
        }
    }
}
