﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using arca4.Scripting;
using arca4.Scripting.JS;
using Ares.Protocol;
using System.Diagnostics.Contracts;

using smnetjs;
namespace arca4
{
    class ServerEvents
    {
		private static Dictionary<EventType, Dictionary<string, SMFunction>> events;

		static ServerEvents()
		{
			events = new Dictionary<EventType, Dictionary<string, SMFunction>>();
			foreach(string eventName in Enum.GetNames(typeof(EventType)))
			{
				events[(EventType)Enum.Parse(typeof(EventType), eventName)] =
					new Dictionary<string, SMFunction>();
			}

		}

		


		

        public static void OnTimer()
        {
			EventManager.InvokeEvent(EventType.OnTimer);
        }

        public static bool OnFlood(UserObject userobj)
        {
			return EventManager.InvokeEvent<bool>(EventType.OnFlood, userobj.ScriptUserObj);
        }

        public static bool OnJoinCheck(UserObject userobj)
        {
			return EventManager.InvokeEvent<bool>(EventType.OnJoinCheck, userobj.ScriptUserObj);
        }

        public static void OnRejected(UserObject userobj, RejectionType e)
        {
			EventManager.InvokeEvent(EventType.OnRejected, userobj.ScriptUserObj, e);
        }

        public static void OnJoin(UserObject userobj)
        {
			EventManager.InvokeEvent(EventType.OnJoin, userobj.ScriptUserObj);
        }

        public static void OnPart(UserObject userobj)
        {
			EventManager.InvokeEvent(EventType.OnPart, userobj.ScriptUserObj);
		}

        public static bool OnAvatarReceived(UserObject userobj)
        {
			return EventManager.InvokeEvent<bool>(EventType.OnAvatarReceived, userobj.ScriptUserObj);
        }

        public static bool OnPersonalMessageReceived(UserObject userobj)
        {
			return EventManager.InvokeEvent<bool>(EventType.OnPersonalMessageReceived, userobj.ScriptUserObj);
        }

        public static String OnTextBefore(UserObject userobj, String text)
        {
            if (userobj.Muzzled)
            {
                userobj.SendPacket(AresTCPPackets.NoSuch("You are muzzled"));
                return String.Empty;
            }


			string ret = EventManager.InvokeEvent<string>(EventType.OnTextBefore, userobj.ScriptUserObj, text);

			return ret == null ? text : ret;
        }

        public static void OnTextAfter(UserObject userobj, String text)
        {

			EventManager.InvokeEvent(EventType.OnTextAfter, userobj.ScriptUserObj, text);
        }

        public static String OnEmoteBefore(UserObject userobj, String text)
        {
            if (userobj.Muzzled)
            {
                userobj.SendPacket(AresTCPPackets.NoSuch("You are muzzled"));
                return String.Empty;
            }

			string ret = EventManager.InvokeEvent<string>(EventType.OnEmoteBefore, userobj.ScriptUserObj, text);

			return ret == null ? text : ret;
        }

        public static void OnEmoteAfter(UserObject userobj, String text)
        {
			EventManager.InvokeEvent(EventType.OnEmoteAfter, userobj.ScriptUserObj, text);
        }

        public static String OnPM(UserObject userobj, UserObject target, String text)
        {
			bool ret = EventManager.InvokeEvent<bool>(EventType.OnPM, userobj.ScriptUserObj, target.ScriptUserObj, text);

            if (!ret)
                return null;

            return text;
        }

        public static void OnBotPM(UserObject userobj, String text)
        {
			EventManager.InvokeEvent(EventType.OnBotPM, userobj.ScriptUserObj, text);
        }

        public static void OnCommand(UserObject userobj, String command, UserObject target, String args, Boolean bot)
        {
            if (command.StartsWith("loadscript "))
            {
                DefaultCommands.LoadScript(userobj, command.Substring(11));
                return;
            }
            else if (command.StartsWith("killscript "))
            {
                DefaultCommands.KillScript(userobj, command.Substring(11));
                return;
            }
            else if (command == "listscripts")
            {
                DefaultCommands.ListScripts(userobj);
                return;
            }

			EventManager.InvokeEvent(EventType.OnCommand, userobj.ScriptUserObj, command, target == null ? null : target.ScriptUserObj, args, bot);

            if (command.StartsWith("ban "))
                DefaultCommands.Ban(userobj, target);
            else if (command.StartsWith("unban "))
                DefaultCommands.Unban(userobj, command.Substring(6));
            else if (command == "listbans")
                DefaultCommands.ListBans(userobj);
            else if (command.StartsWith("muzzle "))
                DefaultCommands.Muzzle(userobj, target);
            else if (command.StartsWith("unmuzzle "))
                DefaultCommands.Unmuzzle(userobj, target);
            else if (command.StartsWith("kill ") || command.StartsWith("kick "))
                DefaultCommands.Kill(userobj, target);
            else if (command.StartsWith("setuserlevel "))
                DefaultCommands.SetUserLevel(userobj, target, args);
            else if (command == "info")
                DefaultCommands.Info(userobj);
        }

        private static String[] PUBLIC_COMMANDS = { "#register <password>", "#unregister <password>", "login <password>", "#info" };

        public static void OnHelp(UserObject userobj, Boolean bot)
        {
			/*  NOT IMPLEMNTING YET IN SCRIPT
			 * TODO: Need a better way to implement the core functions if some of these commands can be overridden in script
			
			*/

            foreach (String command in PUBLIC_COMMANDS)
            {
                userobj.SendPacket((bot ? AresTCPPackets.Private(Settings.BotName, command) : AresTCPPackets.NoSuch(command)));
            }

            if (userobj.Level >= Settings.MuzzleLevel)
            {
                userobj.SendPacket((bot ? AresTCPPackets.Private(Settings.BotName, "#muzzle <user>") : AresTCPPackets.NoSuch("#muzzle <user>")));
                userobj.SendPacket((bot ? AresTCPPackets.Private(Settings.BotName, "#unmuzzle <user>") : AresTCPPackets.NoSuch("#unmuzzle <user>")));
            }

            if (userobj.Level >= Settings.KillLevel)
                userobj.SendPacket((bot ? AresTCPPackets.Private(Settings.BotName, "#kill <user>") : AresTCPPackets.NoSuch("#kill <user>")));

            if (userobj.Level >= Settings.BanLevel)
            {
                userobj.SendPacket((bot ? AresTCPPackets.Private(Settings.BotName, "#ban <user>") : AresTCPPackets.NoSuch("#ban <user>")));
                userobj.SendPacket((bot ? AresTCPPackets.Private(Settings.BotName, "#unban <user>") : AresTCPPackets.NoSuch("#unban <user>")));
                userobj.SendPacket((bot ? AresTCPPackets.Private(Settings.BotName, "#listbans") : AresTCPPackets.NoSuch("#listbans")));
            }

            if (userobj.Level == 3)
                userobj.SendPacket((bot ? AresTCPPackets.Private(Settings.BotName, "#setuserlevel <user> <level>") : AresTCPPackets.NoSuch("#setuserlevel <user> <level>")));

            if (userobj.Level >= Settings.ScriptLevel)
            {
                userobj.SendPacket((bot ? AresTCPPackets.Private(Settings.BotName, "#loadscript <filename>") : AresTCPPackets.NoSuch("#loadscript <filename>")));
                userobj.SendPacket((bot ? AresTCPPackets.Private(Settings.BotName, "#killscript <filename>") : AresTCPPackets.NoSuch("#killscript <filename>")));
                userobj.SendPacket((bot ? AresTCPPackets.Private(Settings.BotName, "#listscripts") : AresTCPPackets.NoSuch("#listscripts")));
            }
		}

        public static bool OnVroomJoinCheck(UserObject userobj, ushort vroom)
        {
			return EventManager.InvokeEvent<bool>(EventType.OnVroomJoinCheck, userobj.ScriptUserObj, vroom);
        }

        public static void OnVroomJoin(UserObject userobj)
        {
			EventManager.InvokeEvent(EventType.OnVroomJoin, userobj.ScriptUserObj);
        }

        public static void OnVroomPart(UserObject userobj)
        {
			EventManager.InvokeEvent(EventType.OnVroomPart, userobj.ScriptUserObj);
        }

        public static void OnFileReceived(UserObject userobj, String filename, String title)
        {
			EventManager.InvokeEvent(EventType.OnFileReceived, userobj.ScriptUserObj, filename, title);
        }

        public static void OnLoginGranted(UserObject userobj)
        {
			EventManager.InvokeEvent(EventType.OnLoginGranted, userobj.ScriptUserObj);
        }

        public static void OnInvalidPassword(UserObject userobj)
        {
			EventManager.InvokeEvent(EventType.OnInvalidPassword, userobj.ScriptUserObj);
        }

    }
}
