﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.Xml.Linq;

namespace arca4
{
    class UserRecordManager
    {
        private static List<UserRecordItem> items;

        static UserRecordManager()
        {
            items = new List<UserRecordItem>();
        }

        public static void AddItem(UserObject userobj, uint time)
        {
            items.Add(new UserRecordItem(userobj, time));
        }

        public static bool IsJoinFlooding(UserObject userobj, uint time)
        {
            return items.FindAll(x => x.ExternalIP.Equals(userobj.ExternalIP) && (x.Time + 15) > time).Count > 0;
        }
    }

    class UserRecordItem
    {
        public String Name { get; private set; }
        public String Version { get; private set; }
        public Guid Guid { get; private set; }
        public IPAddress ExternalIP { get; private set; }
        public IPAddress LocalIP { get; private set; }
        public ushort Port { get; private set; }
        public uint Time { get; private set; }

        public UserRecordItem(UserObject userobj, uint time)
        {
            this.Name = userobj.Name;
            this.Version = userobj.Version;
            this.Guid = userobj.Guid;
            this.ExternalIP = userobj.ExternalIP;
            this.LocalIP = userobj.LocalIP;
            this.Port = userobj.Port;
            this.Time = time;
        }

        public UserRecordItem(XElement xml)
        {
            this.Name = Encoding.UTF8.GetString(Convert.FromBase64String(xml.Element("Name").Value));
            this.Version = Encoding.UTF8.GetString(Convert.FromBase64String(xml.Element("Version").Value));
            this.Guid = new Guid(Convert.FromBase64String(xml.Element("Guid").Value));
            this.ExternalIP = IPAddress.Parse(xml.Element("ExternalIP").Value);
            this.LocalIP = IPAddress.Parse(xml.Element("LocalIP").Value);
            this.Port = ushort.Parse(xml.Element("Port").Value);
            this.Time = uint.Parse(xml.Element("Time").Value);
        }
    }
}
